==> medlogics-common-classes-1.1-20251211.085325-521.jar: com/floreantpos/util/DefaultCOADataInserter.java <==
package com.floreantpos.util;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.hibernate.Session;
import org.hibernate.Transaction;

import com.floreantpos.PosLog;
import com.floreantpos.model.COAAccountType;
import com.floreantpos.model.COAAccountTypeGroup;
import com.floreantpos.model.ChartOfAccounts;
import com.floreantpos.model.DirectionType;
import com.floreantpos.model.dao.COAAccountTypeDAO;
import com.floreantpos.model.dao.COAAccountTypeGroupDAO;
import com.floreantpos.model.dao.ChartOfAccountsDAO;
import com.floreantpos.model.dao.GenericDAO;
import com.floreantpos.model.util.DataProvider;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;

public class DefaultCOADataInserter {

	private static AtomicReference<DataMode> dataMode = new AtomicReference<>(DataMode.MEDLOGICS);
	private String defaultOutletId;

	public DefaultCOADataInserter() {
	}

	public DefaultCOADataInserter(String defaultOutletId, DataMode dataMode) {
		this.defaultOutletId = defaultOutletId;
		this.dataMode.set(dataMode);
	}

	public void insert() {
		Transaction transaction = null;
		try (Session session = GenericDAO.getInstance().createNewSession()) {
			transaction = session.beginTransaction();

			Map<COAAccountTypeGroup, List<COAAccountType>> defaultCOATypeMap = createDefaultCOAType();
			for (Map.Entry<COAAccountTypeGroup, List<COAAccountType>> entry : defaultCOATypeMap.entrySet()) {
				COAAccountTypeGroup group = entry.getKey();
				List<COAAccountType> types = entry.getValue();

				COAAccountTypeGroupDAO.getInstance().save(group, session);
				for (COAAccountType coaAccountType : types) {
					COAAccountTypeDAO.getInstance().save(coaAccountType, session);
				}
			}

			List<ChartOfAccounts> chartOfAccounts = defaultCOAData();
			for (ChartOfAccounts chartOfAccount : chartOfAccounts) {
				chartOfAccount.putDefaultCoa(true);
				ChartOfAccountsDAO.getInstance().save(chartOfAccount, session);
			}

			transaction.commit();
		} catch (Exception e) {
			PosLog.error(getClass(), e);
		}
	}

	private static Map<COAAccountTypeGroup, List<COAAccountType>> createDefaultCOAType() {
		Map<COAAccountTypeGroup, List<COAAccountType>> coaMap = new HashMap<>();
		//Asset group
		COAAccountTypeGroup assetGroup = new COAAccountTypeGroup("asset", "Asset");
		assetGroup.setAccountDirection(DirectionType.DEBIT.getTypeNo());

		COAAccountType currentAssetType = new COAAccountType("current_asset", "Current asset");
		currentAssetType.setCoaAccountTypeGroupId("asset");

		COAAccountType fixedAssetType = new COAAccountType("fixed_asset", "Fixed asset");
		fixedAssetType.setCoaAccountTypeGroupId("asset");

		COAAccountType inventoryType = new COAAccountType("inventory", "Inventory");
		inventoryType.setCoaAccountTypeGroupId("asset");

		COAAccountType nonCurrentAssetType = new COAAccountType("non_current_asset", "Non-current asset");
		nonCurrentAssetType.setCoaAccountTypeGroupId("asset");

		COAAccountType prepaymentType = new COAAccountType("prepayment", "Prepayment");
		prepaymentType.setCoaAccountTypeGroupId("asset");

		coaMap.put(assetGroup, Arrays.asList(currentAssetType, fixedAssetType, inventoryType, nonCurrentAssetType, prepaymentType));

		//Equity group
		COAAccountTypeGroup equityGroup = new COAAccountTypeGroup("equity", "Equity");
		equityGroup.setAccountDirection(DirectionType.CREDIT.getTypeNo());

		COAAccountType equityType = new COAAccountType("equity", "Equity");
		equityType.setCoaAccountTypeGroupId("equity");

		coaMap.put(equityGroup, Arrays.asList(equityType));

		//Expense group
		COAAccountTypeGroup expenseGroup = new COAAccountTypeGroup("expense", "Expense");
		expenseGroup.setAccountDirection(DirectionType.DEBIT.getTypeNo());

		COAAccountType depreciationType = new COAAccountType("depreciation", "Depreciation");
		depreciationType.setCoaAccountTypeGroupId("expense");

		COAAccountType directCostsType = new COAAccountType("direct_costs", "Direct costs");
		directCostsType.setCoaAccountTypeGroupId("expense");

		COAAccountType expenseType = new COAAccountType("expense", "Expense");
		expenseType.setCoaAccountTypeGroupId("expense");

		COAAccountType overheadType = new COAAccountType("overhead", "Overhead");
		overheadType.setCoaAccountTypeGroupId("expense");

		coaMap.put(expenseGroup, Arrays.asList(depreciationType, directCostsType, expenseType, overheadType));

		//Liabilities group
		COAAccountTypeGroup liabilitiesGroup = new COAAccountTypeGroup("liabilities", "Liabilities");
		liabilitiesGroup.setAccountDirection(DirectionType.CREDIT.getTypeNo());

		COAAccountType currentLiabilityType = new COAAccountType("current_liability", "Current liability");
		currentLiabilityType.setCoaAccountTypeGroupId("liabilities");

		COAAccountType liabilityType = new COAAccountType("liability", "Liability");
		liabilityType.setCoaAccountTypeGroupId("liabilities");

		COAAccountType nonCurrentLiabilityType = new COAAccountType("non_current_liability", "Non-current liability");
		nonCurrentLiabilityType.setCoaAccountTypeGroupId("liabilities");

		coaMap.put(liabilitiesGroup, Arrays.asList(currentLiabilityType, liabilityType, nonCurrentLiabilityType));

		//Revenue group
		COAAccountTypeGroup revenueGroup = new COAAccountTypeGroup("revenue", "Revenue");
		revenueGroup.setAccountDirection(DirectionType.CREDIT.getTypeNo());

		COAAccountType otherIncomeType = new COAAccountType("other_income", "Other income");
		otherIncomeType.setCoaAccountTypeGroupId("revenue");

		COAAccountType revenueType = new COAAccountType("revenue", "Revenue");
		revenueType.setCoaAccountTypeGroupId("revenue");

		COAAccountType salesType = new COAAccountType("sales", "Sales");
		salesType.setCoaAccountTypeGroupId("revenue");

		COAAccountType contraAccountType = new COAAccountType("contra_account", "Contra account");
		contraAccountType.setCoaAccountTypeGroupId("revenue");

		coaMap.put(revenueGroup, Arrays.asList(otherIncomeType, revenueType, salesType, contraAccountType));

		return coaMap;
	}

	private static List<ChartOfAccounts> defaultCOAData() {
		List<ChartOfAccounts> chartOfAccounts = Collections.emptyList();

		//		DataMode dataMode = DataProvider.get().getDataMode();
		//		String fileName = "/ChartOfAccounts.json";
		//		if (dataMode == DataMode.BIZLOGICS) {
		//			fileName = "/ChartOfAccounts_biz.json";
		//		}
		//		else if (dataMode == DataMode.BIZLOGICS_COA_V2) {
		//			fileName = "/ChartOfAccounts_v2_biz.json";
		//		}

		DataMode dataMode = DataProvider.get().getDataMode();
		String fileName = "/ChartOfAccounts_v2_med.json";
		if (dataMode == DataMode.BIZLOGICS || dataMode == DataMode.BIZLOGICS_COA_V2) {
			fileName = "/ChartOfAccounts_v2_biz.json";
		}

		try (InputStream resourceAsStream = DefaultCOADataInserter.class.getResourceAsStream(fileName)) { //$NON-NLS-1$
			if (resourceAsStream != null) {
				Type listType = new TypeToken<ArrayList<ChartOfAccounts>>() {
				}.getType();

				chartOfAccounts = new Gson().fromJson(IOUtils.toString(resourceAsStream), listType);
			}
		} catch (IOException e) {
			PosLog.error(DefaultCOADataInserter.class, e);
		}
		return chartOfAccounts;
	}

	public static ChartOfAccounts getChartOfAccountsByCode(String code) {
		ChartOfAccounts chartOfAccounts = defaultCOAData().stream().filter(coa -> coa.getAccountCode().equals(code)).findFirst().orElse(null);
		saveChartOfAccounts(chartOfAccounts);
		return chartOfAccounts;
	}

	private static void saveChartOfAccounts(ChartOfAccounts chartOfAccounts) {
		if (chartOfAccounts != null) {
			chartOfAccounts.putDefaultCoa(true);
			Map<COAAccountTypeGroup, List<COAAccountType>> defaultCOAType = createDefaultCOAType();
			Transaction transaction = null;
			try (Session session = GenericDAO.getInstance().createNewSession()) {
				transaction = session.beginTransaction();

				defaultCOAType.forEach((group, types) -> {
					for (COAAccountType coaAccountType : types) {
						if (coaAccountType.getId().equals(chartOfAccounts.getCoaAccountTypeId())) {

							if (session.get(COAAccountTypeGroup.class, group.getId()) == null) {
								COAAccountTypeGroupDAO.getInstance().save(group, session);
							}
							if (session.get(COAAccountType.class, coaAccountType.getId()) == null) {
								COAAccountTypeDAO.getInstance().save(coaAccountType, session);
							}

							return;
						}
					}
					return;
				});

				if (StringUtils.isBlank(chartOfAccounts.getId())) {
					ChartOfAccountsDAO.getInstance().save(chartOfAccounts, session);
				}

				transaction.commit();
			} catch (Exception e) {
				if (transaction != null) {
					transaction.rollback();
				}
				PosLog.error(DefaultCOADataInserter.class, e);
			}
		}
	}

	public enum DataMode {
		MEDLOGICS, BIZLOGICS, BIZLOGICS_COA_V2
	}
}

==> medlogics-common-classes-1.4.7.jar: com/floreantpos/util/DefaultCOADataInserter.java <==
package com.floreantpos.util;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.hibernate.Session;
import org.hibernate.Transaction;

import com.floreantpos.PosLog;
import com.floreantpos.model.COAAccountType;
import com.floreantpos.model.COAAccountTypeGroup;
import com.floreantpos.model.ChartOfAccounts;
import com.floreantpos.model.DirectionType;
import com.floreantpos.model.dao.COAAccountTypeDAO;
import com.floreantpos.model.dao.COAAccountTypeGroupDAO;
import com.floreantpos.model.dao.ChartOfAccountsDAO;
import com.floreantpos.model.dao.GenericDAO;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;

public class DefaultCOADataInserter {

	private String defaultOutletId;

	public DefaultCOADataInserter() {
	}
	
	public DefaultCOADataInserter(String defaultOutletId) {
		this.defaultOutletId = defaultOutletId;
	}

	public void insert() {
		Transaction transaction = null;
		try (Session session = GenericDAO.getInstance().createNewSession()) {
			transaction = session.beginTransaction();

			Map<COAAccountTypeGroup, List<COAAccountType>> defaultCOATypeMap = createDefaultCOAType();
			for (Map.Entry<COAAccountTypeGroup, List<COAAccountType>> entry : defaultCOATypeMap.entrySet()) {
				COAAccountTypeGroup group = entry.getKey();
				List<COAAccountType> types = entry.getValue();

				COAAccountTypeGroupDAO.getInstance().save(group, session);
				for (COAAccountType coaAccountType : types) {
					COAAccountTypeDAO.getInstance().save(coaAccountType, session);
				}
			}

			List<ChartOfAccounts> chartOfAccounts = defaultCOAData();
			for (ChartOfAccounts chartOfAccount : chartOfAccounts) {
				chartOfAccount.putDefaultCoa(true);
				ChartOfAccountsDAO.getInstance().save(chartOfAccount, session);
			}

			transaction.commit();
		} catch (Exception e) {
			PosLog.error(getClass(), e);
		}
	}

	private Map<COAAccountTypeGroup, List<COAAccountType>> createDefaultCOAType() {
		Map<COAAccountTypeGroup, List<COAAccountType>> coaMap = new HashMap<>();
		//Asset group
		COAAccountTypeGroup assetGroup = new COAAccountTypeGroup("asset", "Asset");
		assetGroup.setAccountDirection(DirectionType.DEBIT.getTypeNo());

		COAAccountType currentAssetType = new COAAccountType("current_asset", "Current asset");
		currentAssetType.setCoaAccountTypeGroupId("asset");

		COAAccountType fixedAssetType = new COAAccountType("fixed_asset", "Fixed asset");
		fixedAssetType.setCoaAccountTypeGroupId("asset");

		COAAccountType inventoryType = new COAAccountType("inventory", "Inventory");
		inventoryType.setCoaAccountTypeGroupId("asset");

		COAAccountType nonCurrentAssetType = new COAAccountType("non_current_asset", "Non-current asset");
		nonCurrentAssetType.setCoaAccountTypeGroupId("asset");

		COAAccountType prepaymentType = new COAAccountType("prepayment", "Prepayment");
		prepaymentType.setCoaAccountTypeGroupId("asset");

		coaMap.put(assetGroup, Arrays.asList(currentAssetType, fixedAssetType, inventoryType, nonCurrentAssetType, prepaymentType));

		//Equity group
		COAAccountTypeGroup equityGroup = new COAAccountTypeGroup("equity", "Equity");
		equityGroup.setAccountDirection(DirectionType.CREDIT.getTypeNo());

		COAAccountType equityType = new COAAccountType("equity", "Equity");
		equityType.setCoaAccountTypeGroupId("equity");

		coaMap.put(equityGroup, Arrays.asList(equityType));

		//Expense group
		COAAccountTypeGroup expenseGroup = new COAAccountTypeGroup("expense", "Expense");
		expenseGroup.setAccountDirection(DirectionType.DEBIT.getTypeNo());

		COAAccountType depreciationType = new COAAccountType("depreciation", "Depreciation");
		depreciationType.setCoaAccountTypeGroupId("expense");

		COAAccountType directCostsType = new COAAccountType("direct_costs", "Direct costs");
		directCostsType.setCoaAccountTypeGroupId("expense");

		COAAccountType expenseType = new COAAccountType("expense", "Expense");
		expenseType.setCoaAccountTypeGroupId("expense");

		COAAccountType overheadType = new COAAccountType("overhead", "Overhead");
		overheadType.setCoaAccountTypeGroupId("expense");

		coaMap.put(expenseGroup, Arrays.asList(depreciationType, directCostsType, expenseType, overheadType));

		//Liabilities group
		COAAccountTypeGroup liabilitiesGroup = new COAAccountTypeGroup("liabilities", "Liabilities");
		liabilitiesGroup.setAccountDirection(DirectionType.CREDIT.getTypeNo());

		COAAccountType currentLiabilityType = new COAAccountType("current_liability", "Current liability");
		currentLiabilityType.setCoaAccountTypeGroupId("liabilities");

		COAAccountType liabilityType = new COAAccountType("liability", "Liability");
		liabilityType.setCoaAccountTypeGroupId("liabilities");

		COAAccountType nonCurrentLiabilityType = new COAAccountType("non_current_liability", "Non-current liability");
		nonCurrentLiabilityType.setCoaAccountTypeGroupId("liabilities");

		coaMap.put(liabilitiesGroup, Arrays.asList(currentLiabilityType, liabilityType, nonCurrentLiabilityType));

		//Revenue group
		COAAccountTypeGroup revenueGroup = new COAAccountTypeGroup("revenue", "Revenue");
		revenueGroup.setAccountDirection(DirectionType.CREDIT.getTypeNo());

		COAAccountType otherIncomeType = new COAAccountType("other_income", "Other income");
		otherIncomeType.setCoaAccountTypeGroupId("revenue");

		COAAccountType revenueType = new COAAccountType("revenue", "Revenue");
		revenueType.setCoaAccountTypeGroupId("revenue");

		COAAccountType salesType = new COAAccountType("sales", "Sales");
		salesType.setCoaAccountTypeGroupId("revenue");

		coaMap.put(revenueGroup, Arrays.asList(otherIncomeType, revenueType, salesType));

		return coaMap;
	}

	private static List<ChartOfAccounts> defaultCOAData() {
		List<ChartOfAccounts> chartOfAccounts = Collections.emptyList();
		try (InputStream resourceAsStream = DefaultCOADataInserter.class.getResourceAsStream("/ChartOfAccounts.json")) { //$NON-NLS-1$
			if (resourceAsStream != null) {
				Type listType = new TypeToken<ArrayList<ChartOfAccounts>>() {
				}.getType();

				chartOfAccounts = new Gson().fromJson(IOUtils.toString(resourceAsStream), listType);
			}
		} catch (IOException e) {
			PosLog.error(DefaultCOADataInserter.class, e);
		}
		return chartOfAccounts;
	}
	

	public static ChartOfAccounts getChartOfAccountsByCode(String code) {
		ChartOfAccounts chartOfAccounts = defaultCOAData().stream().filter(coa -> coa.getAccountCode().equals(code)).findFirst().orElse(null);
		saveChartOfAccounts(chartOfAccounts);
		return chartOfAccounts;
	}

	private static void saveChartOfAccounts(ChartOfAccounts chartOfAccounts) {
		if (chartOfAccounts != null) {
			chartOfAccounts.putDefaultCoa(true);
			Map<COAAccountTypeGroup, List<COAAccountType>> defaultCOAType = new DefaultCOADataInserter().createDefaultCOAType();
			Transaction transaction = null;
			try (Session session = GenericDAO.getInstance().createNewSession()) {
				transaction = session.beginTransaction();

				defaultCOAType.forEach((group, types) -> {
					for (COAAccountType coaAccountType : types) {
						if (coaAccountType.getId().equals(chartOfAccounts.getCoaAccountTypeId())) {

							if (session.get(COAAccountTypeGroup.class, group.getId()) == null) {
								COAAccountTypeGroupDAO.getInstance().save(group, session);
							}
							if (session.get(COAAccountType.class, coaAccountType.getId()) == null) {
								COAAccountTypeDAO.getInstance().save(coaAccountType, session);
							}

							return;
						}
					}
					return;
				});
				
				if (StringUtils.isBlank(chartOfAccounts.getId())) {
					ChartOfAccountsDAO.getInstance().save(chartOfAccounts, session);
				}

				transaction.commit();
			} catch (Exception e) {
				if (transaction != null) {
					transaction.rollback();
				}
				PosLog.error(DefaultCOADataInserter.class, e);
			}
		}
	}

}
